// http://baagoe.org/en/wiki/Better_random_numbers_for_javascript
// Copyright (C) 2010 by Johannes Baagøe <baagoe@baagoe.org>
// MIT License - http://en.wikipedia.org/wiki/MIT_License
// Modified

/*jshint boss:true */

function Mash() {
	var n = 0xefc8249d;

	var mash = function(data) {
		data = data.toString();
		for (var i = 0; i < data.length; i++) {
			n += data.charCodeAt(i);
			var h = 0.02519603282416938 * n;
			n = h >>> 0;
			h -= n;
			h *= n;
			n = h >>> 0;
			h -= n;
			n += h * 0x100000000; // 2^32
		}
		return (n >>> 0) * 2.3283064365386963e-10; // 2^-32
	};

	mash.version = 'Mash 0.9';
	return mash;
}

function Alea() {
	var args = Array.prototype.slice.call(arguments);
	// Johannes Baagøe <baagoe@baagoe.com>, 2010
	var s0 = 0;
	var s1 = 0;
	var s2 = 0;
	var c = 1;

	if (args.length === 0) {
		args = [+new Date()];
	}
	var mash = Mash();
	s0 = mash(' ');
	s1 = mash(' ');
	s2 = mash(' ');

	for (var i = 0; i < args.length; i++) {
		s0 -= mash(args[i]);
		if (s0 < 0) {
			s0 += 1;
		}
		s1 -= mash(args[i]);
		if (s1 < 0) {
			s1 += 1;
		}
		s2 -= mash(args[i]);
		if (s2 < 0) {
			s2 += 1;
		}
	}
	mash = null;

	this.random = function() {
		var t = 2091639 * s0 + c * 2.3283064365386963e-10; // 2^-32
		s0 = s1;
		s1 = s2;
		return s2 = t - (c = t | 0);
	};
	this.random.uint32 = function() {
		return random() * 0x100000000; // 2^32
	};
	this.random.fract53 = function() {
		return random() + 
			(random() * 0x200000 | 0) * 1.1102230246251565e-16; // 2^-53
	};
	this.random.version = 'Alea 0.9';
	this.random.args = args;
}
